import { MinecraftDimensionTypes, world } from '@minecraft/server';
import { ConfigScoreboardsObjectives, ConfigScoreboardsParticipants, mapStarted } from 'config/scoreboards';
import { AvaliableRange } from 'config/variables';
import { addParticipant, getScore, setScore } from 'scoreboard/scoreboard';
import { getRandomInt } from 'utils';
const mobs = [
    {
        typeId: [
            'minecraft:pig',
            'minecraft:cow',
            'minecraft:sheep',
            'minecraft:zombie'
        ],
        level: 1,
        requiredProgress: 10
    },
    {
        typeId: ['minecraft:llama', 'minecraft:llama', 'minecraft:zombie'],
        level: 2,
        requiredProgress: 10
    },
    {
        typeId: ['minecraft:goat', 'minecraft:goat', 'minecraft:zombie'],
        level: 3,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:zombie',
        level: 4,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:vindicator',
        level: 5,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:silverfish',
        level: 6,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:pillager',
        level: 7,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:piglin_brute',
        level: 8,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:zoglin',
        level: 9,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:ravager',
        level: 10,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:enderman',
        level: 11,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:phantom',
        level: 12,
        requiredProgress: 10
    },
    {
        typeId: 'minecraft:iron_golem',
        level: 13,
        requiredProgress: 10
    }
];
function randomCoordinates(arena) {
    return {
        x: getRandomInt(AvaliableRange.arenas[arena].x.x1, AvaliableRange.arenas[arena].x.x2),
        z: getRandomInt(AvaliableRange.arenas[arena].z.z1, AvaliableRange.arenas[arena].z.z2)
    };
}
export function startFirstWave() {
    ;
    [...new Array(mobs[0].requiredProgress)].forEach(() => {
        const currentArena = getScore({
            objectiveId: ConfigScoreboardsObjectives.Config,
            participantName: ConfigScoreboardsParticipants.SelectedMap
        });
        const { x, z } = randomCoordinates(currentArena);
        const currentMob = mobs.find((mob) => mob.level === 1);
        if (!currentMob) {
            world.sendMessage('§4error in wave file');
            return;
        }
        let { typeId } = currentMob;
        if (typeof typeId !== 'string') {
            typeId = typeId[getRandomInt(0, typeId.length - 1)];
        }
        spawnEntity(typeId, currentArena, {
            x,
            y: AvaliableRange.arenas[currentArena].y,
            z
        });
    });
}
world.afterEvents.entityDie.subscribe((event) => {
    const { deadEntity, damageSource } = event;
    const excludeEntities = ['aa:barrel_tnt', 'aa:lucky_parachute'];
    if (!mapStarted() || excludeEntities.includes(event.deadEntity.typeId))
        return;
    const currentArena = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.SelectedMap
    });
    if (damageSource?.damagingEntity?.typeId === 'minecraft:player')
        addKilledScore(damageSource?.damagingEntity);
    const overworld = world.getDimension(MinecraftDimensionTypes.overworld);
    let level = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.Level
    });
    let progress = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.Progress
    });
    if (level === null || progress === null)
        return;
    let currentMobLevel = mobs.find((mob) => mob.level === level);
    const mobIsFromCurrentLevel = currentMobLevel &&
        (deadEntity.typeId === currentMobLevel.typeId ||
            currentMobLevel.typeId.includes(deadEntity.typeId));
    const IsLuckyMob = deadEntity.typeId === 'aa:lucky_mob';
    if (mobIsFromCurrentLevel || IsLuckyMob) {
        const actualProgress = progress + 1;
        const updateLevel = actualProgress === currentMobLevel.requiredProgress;
        if (updateLevel) {
            level++;
            progress = 0;
            setScore({
                objectiveId: ConfigScoreboardsObjectives.Config,
                participant: {
                    name: ConfigScoreboardsParticipants.Progress,
                    value: progress
                }
            });
            setScore({
                objectiveId: ConfigScoreboardsObjectives.Config,
                participant: {
                    name: ConfigScoreboardsParticipants.Level,
                    value: level
                }
            });
            const hasNextLevel = mobs.find((mob) => mob.level === level);
            if (!hasNextLevel) {
                //* Win
                overworld.runCommand('/title @a title §2You Win!!');
                overworld.runCommand('/scriptevent aa:map_situation stop');
                return;
            }
            overworld.runCommand('/title @a title §2Level Up');
            overworld.runCommand(`/title @a subtitle Level: ${level} `);
            overworld.runCommand('/playsound random.levelup @a');
        }
        else {
            progress++;
            setScore({
                objectiveId: ConfigScoreboardsObjectives.Config,
                participant: {
                    name: ConfigScoreboardsParticipants.Progress,
                    value: progress
                }
            });
        }
    }
    else {
        //? mob died but is not  the current wave
        if (!currentMobLevel)
            return;
        const isLastLevel = currentMobLevel.level === mobs[mobs.length - 1].level;
        if (isLastLevel)
            return;
    }
    let luckyMobScore = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.LuckyMob
    });
    luckyMobScore++;
    const randomNumber = getRandomInt(1, 100);
    currentMobLevel = mobs.find((mob) => mob.level === level);
    if (!currentMobLevel)
        return;
    let { typeId } = currentMobLevel;
    if (typeof typeId !== 'string') {
        typeId = typeId[getRandomInt(0, typeId.length - 1)];
    }
    const { x, z } = randomCoordinates(currentArena);
    if (luckyMobScore >= randomNumber) {
        //? Spawn lucky mob
        world.sendMessage('§aLucky Mob');
        spawnEntity('aa:lucky_mob', currentArena, {
            x,
            y: AvaliableRange.arenas[currentArena].y,
            z
        });
        setScore({
            objectiveId: ConfigScoreboardsObjectives.Config,
            participant: {
                name: ConfigScoreboardsParticipants.LuckyMob,
                value: 0
            }
        });
    }
    else {
        spawnEntity(typeId, currentArena, {
            x,
            y: AvaliableRange.arenas[currentArena].y,
            z
        });
        setScore({
            objectiveId: ConfigScoreboardsObjectives.Config,
            participant: {
                name: ConfigScoreboardsParticipants.LuckyMob,
                value: luckyMobScore
            }
        });
    }
});
function addKilledScore(player) {
    const score = getScore({
        objectiveId: ConfigScoreboardsObjectives.MobsKilled,
        participantName: player.name
    });
    if (score === null) {
        addParticipant({
            objectiveId: ConfigScoreboardsObjectives.MobsKilled,
            participant: {
                name: player.name,
                value: 1
            }
        });
    }
    else
        setScore({
            objectiveId: ConfigScoreboardsObjectives.MobsKilled,
            participant: {
                name: player.name,
                value: score + 1
            }
        });
}
function spawnEntity(entityTypeId, currentArena, location) {
    const overworld = world.getDimension(MinecraftDimensionTypes.overworld);
    let { x, z, y } = location;
    const block = overworld.getBlock({
        x,
        y,
        z
    });
    if (block.isLiquid) {
        const { x: newX, z: newZ } = randomCoordinates(currentArena);
        spawnEntity(entityTypeId, currentArena, { y, x: newX, z: newZ });
        return;
    }
    if (!block.isAir) {
        y++;
        const newBlock = overworld.getBlock({
            x,
            y,
            z
        });
        if (!newBlock.isAir) {
            const { x: newX, z: newZ } = randomCoordinates(currentArena);
            spawnEntity(entityTypeId, currentArena, { y, x: newX, z: newZ });
            return;
        }
    }
    const entity = overworld.spawnEntity(entityTypeId, { x, y, z });
    entity.addTag('wave_mob');
}
