import { world } from '@minecraft/server';
export function initializeObjectives({ objective, participants }) {
    const existObjectId = world.scoreboard.getObjective(objective.id);
    if (!existObjectId) {
        const objectiveScoreboard = world.scoreboard.addObjective(objective.id, objective.displayName);
        participants.forEach((score) => objectiveScoreboard.addScore(score.name, score.value));
    }
}
export function getScore({ objectiveId, participantName }) {
    const objective = world.scoreboard.getObjective(objectiveId);
    if (!objective)
        return null;
    if (!objective.hasParticipant(participantName))
        return null;
    const score = objective.getScore(participantName);
    if (typeof score === 'undefined')
        return null;
    return score;
}
export function setScore({ objectiveId, participant }) {
    const objective = world.scoreboard.getObjective(objectiveId);
    if (!objective)
        return;
    objective.setScore(participant.name, participant.value);
}
export function addParticipant({ objectiveId, participant }) {
    const objective = world.scoreboard.getObjective(objectiveId);
    if (!objective)
        return;
    objective.addScore(participant.name, participant.value);
}
export function removeParticipant({ objectiveId, participantName }) {
    const objective = world.scoreboard.getObjective(objectiveId);
    if (!objective)
        return;
    objective.removeParticipant(participantName);
}
export function removeAllParticipant({ objectiveId }) {
    const objective = world.scoreboard.getObjective(objectiveId);
    if (!objective)
        return;
    const participants = objective.getParticipants();
    participants.forEach((participant) => objective.removeParticipant(participant.displayName));
}
