import { EquipmentSlot, ItemStack, MinecraftDimensionTypes, system, world } from '@minecraft/server';
import { getScore, removeAllParticipant, setScore } from '../scoreboard/scoreboard';
import { ConfigScoreboardsObjectives, ConfigStartedScoreValues, ConfigScoreboardsParticipants, ConfigDifficultyScoreValues } from '../config/scoreboards';
import { startFirstWave } from 'features/wave/wave';
import { PlayerTags } from '../config/tags';
import { ArenaLocation, AvaliableRange, LobbyLocation } from 'config/variables';
export function mapSituation(message) {
    if (message === 'start') {
        startMap();
    }
    else if (message === 'stop') {
        stopMap();
    }
    else {
        world.sendMessage('eventScript with unknow message');
        return;
    }
}
function startMap() {
    const currentArena = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.SelectedMap
    });
    world.getAllPlayers().forEach((player) => {
        player.teleport(ArenaLocation[currentArena]);
        player.triggerEvent('aa:pvp');
    });
    setScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participant: {
            name: ConfigScoreboardsParticipants.MapStarted,
            value: ConfigStartedScoreValues.True
        }
    });
    const overworld = world.getDimension(MinecraftDimensionTypes.overworld);
    overworld.runCommand(`/spawnpoint @a ${ArenaLocation[currentArena].x} ${ArenaLocation[currentArena].y} ${ArenaLocation[currentArena].z}`);
    system.runTimeout(() => {
        startFirstWave();
    }, 10);
    getEquipaments();
    setDifficultily();
}
function stopMap() {
    const currentArena = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.SelectedMap
    });
    world.getAllPlayers().forEach((player) => {
        player.teleport(LobbyLocation);
        player.triggerEvent('aa:no_pvp');
    });
    const overworld = world.getDimension(MinecraftDimensionTypes.overworld);
    overworld.runCommand(`/spawnpoint @a ${LobbyLocation.x} ${LobbyLocation.y} ${LobbyLocation.z}`);
    overworld.runCommand('/clear @a');
    overworld.runCommand('/kill @e[tag=wave_mob]');
    system.runTimeout(() => {
        overworld.runCommand('/kill @e[type=item]');
        overworld.runCommand('/kill @e[type=xp_orb]');
    }, 20);
    const { x: { x1, x2 }, y, z: { z1, z2 } } = AvaliableRange.arenas[currentArena];
    overworld.runCommand(`/fill ${x1} ${y - 2} ${z1} ${x2} ${y + 10} ${z2} air replace aa:lucky_block`);
    resetScoreboards();
    resetPlayerTags();
}
function setDifficultily() {
    const overworld = world.getDimension(MinecraftDimensionTypes.overworld);
    let triggerPlayerEvent;
    const difficultily = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.Difficulty
    });
    switch (difficultily) {
        case ConfigDifficultyScoreValues.Hard:
            overworld.runCommand('/fog @a remove b');
            overworld.runCommand('/time set day');
            triggerPlayerEvent = 'aa:hard_mode';
            break;
        case ConfigDifficultyScoreValues.Medium:
            overworld.runCommand('/fog @a remove b');
            overworld.runCommand('/time set night');
            triggerPlayerEvent = 'aa:medium_mode';
            break;
        case ConfigDifficultyScoreValues.Easy:
            overworld.runCommand('/fog @a push minecraft:fog_crimson_forest b');
            overworld.runCommand('/time set night');
            triggerPlayerEvent = 'aa:easy_mode';
            break;
    }
    world
        .getAllPlayers()
        .forEach((player) => player.triggerEvent(triggerPlayerEvent));
}
function resetPlayerTags() {
    world.getAllPlayers().forEach((player) => player.removeTag(PlayerTags.Died));
}
function resetScoreboards() {
    setScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participant: {
            name: ConfigScoreboardsParticipants.MapStarted,
            value: ConfigStartedScoreValues.False
        }
    });
    setScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participant: {
            name: ConfigScoreboardsParticipants.Timer,
            value: 10
        }
    });
    setScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participant: {
            name: ConfigScoreboardsParticipants.Progress,
            value: 0
        }
    });
    setScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participant: {
            name: ConfigScoreboardsParticipants.Level,
            value: 1
        }
    });
    setScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participant: {
            name: ConfigScoreboardsParticipants.LuckyMob,
            value: 0
        }
    });
    removeAllParticipant({
        objectiveId: ConfigScoreboardsObjectives.PlayersDied
    });
}
export function getEquipaments(playerId) {
    const difficultily = getScore({
        objectiveId: ConfigScoreboardsObjectives.Config,
        participantName: ConfigScoreboardsParticipants.Difficulty
    });
    if (difficultily === null)
        return;
    let startSword;
    let startPickaxe = new ItemStack('netherite_pickaxe', 1);
    startPickaxe.setCanDestroy(['aa:lucky_block']);
    let startFood = new ItemStack('cooked_beef', 64);
    let startArmorHelmet;
    let startArmorChestplate;
    let startArmorLeggings;
    let startArmorBoots;
    switch (difficultily) {
        case ConfigDifficultyScoreValues.Easy:
            startSword = new ItemStack('minecraft:iron_sword', 1);
            startArmorHelmet = new ItemStack('minecraft:chainmail_helmet', 1);
            startArmorChestplate = new ItemStack('minecraft:chainmail_chestplate', 1);
            startArmorLeggings = new ItemStack('minecraft:chainmail_leggings', 1);
            startArmorBoots = new ItemStack('minecraft:chainmail_boots', 1);
            break;
        case ConfigDifficultyScoreValues.Medium:
            startSword = new ItemStack('minecraft:iron_sword', 1);
            startArmorHelmet = new ItemStack('minecraft:leather_helmet', 1);
            startArmorChestplate = new ItemStack('minecraft:leather_chestplate', 1);
            startArmorLeggings = new ItemStack('minecraft:leather_leggings', 1);
            startArmorBoots = new ItemStack('minecraft:leather_boots', 1);
            break;
        case ConfigDifficultyScoreValues.Hard:
            startSword = new ItemStack('minecraft:stone_sword', 1);
            break;
    }
    if (playerId) {
        const player = world
            .getAllPlayers()
            .find((player) => player.id === playerId);
        const equippable = player.getComponent('minecraft:equippable');
        const inventory = player.getComponent('minecraft:inventory');
        inventory.container.addItem(startSword);
        inventory.container.addItem(startFood);
        inventory.container.addItem(startPickaxe);
        equippable.setEquipment(EquipmentSlot.Head, startArmorHelmet);
        equippable.setEquipment(EquipmentSlot.Chest, startArmorChestplate);
        equippable.setEquipment(EquipmentSlot.Legs, startArmorLeggings);
        equippable.setEquipment(EquipmentSlot.Feet, startArmorBoots);
    }
    else {
        world.getAllPlayers().forEach((player, index) => {
            const equippable = player.getComponent('minecraft:equippable');
            const inventory = player.getComponent('minecraft:inventory');
            inventory.container.addItem(startSword);
            inventory.container.addItem(startFood);
            inventory.container.addItem(startPickaxe);
            equippable.setEquipment(EquipmentSlot.Head, startArmorHelmet);
            equippable.setEquipment(EquipmentSlot.Chest, startArmorChestplate);
            equippable.setEquipment(EquipmentSlot.Legs, startArmorLeggings);
            equippable.setEquipment(EquipmentSlot.Feet, startArmorBoots);
        });
    }
}
